
# Imports
from beet.core.utils import JsonDict

# This file is auto-generated by the release script (python_package/scripts/on_bookshelf_release.py).
# Do not edit this file manually.
BOOKSHELF_MODULES: dict[str, JsonDict] = {
	"bs.bitwise": {"version": [3,2,0],"name": "Bookshelf Bitwise","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.block": {"version": [3,2,0],"name": "Bookshelf Block","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.color": {"version": [3,2,0],"name": "Bookshelf Color","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.dump": {"version": [3,2,0],"name": "Bookshelf Dump","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.environment": {"version": [3,2,0],"name": "Bookshelf Environment","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.generation": {"version": [3,2,0],"name": "Bookshelf Generation","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.health": {"version": [3,2,0],"name": "Bookshelf Health","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.hitbox": {"version": [3,2,0],"name": "Bookshelf Hitbox","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.id": {"version": [3,2,0],"name": "Bookshelf Id","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.interaction": {"version": [3,2,0],"name": "Bookshelf Interaction","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.link": {"version": [3,2,0],"name": "Bookshelf Link","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.log": {"version": [3,2,0],"name": "Bookshelf Log","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.math": {"version": [3,2,0],"name": "Bookshelf Math","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.move": {"version": [3,2,0],"name": "Bookshelf Move","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.position": {"version": [3,2,0],"name": "Bookshelf Position","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.random": {"version": [3,2,0],"name": "Bookshelf Random","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.raycast": {"version": [3,2,0],"name": "Bookshelf Raycast","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.schedule": {"version": [3,2,0],"name": "Bookshelf Schedule","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.sidebar": {"version": [3,2,0],"name": "Bookshelf Sidebar","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.spline": {"version": [3,2,0],"name": "Bookshelf Spline","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.string": {"version": [3,2,0],"name": "Bookshelf String","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.time": {"version": [3,2,0],"name": "Bookshelf Time","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.tree": {"version": [3,2,0],"name": "Bookshelf Tree","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.vector": {"version": [3,2,0],"name": "Bookshelf Vector","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.view": {"version": [3,2,0],"name": "Bookshelf View","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False},
	"bs.xp": {"version": [3,2,0],"name": "Bookshelf Xp","url": "https://github.com/mcbookshelf/bookshelf/releases","is_used": False}
}

